package app.data.service;

import app.data.chesscom.ChessComArchive;
import app.data.chesscom.ChessComArchiveList;
import app.data.chesscom.ChessComGame;
import app.data.entity.Game;
import app.data.entity.GameInfo;
import app.data.entity.Match;
import app.data.entity.Player;
import app.utils.ChessComUtils;
import app.utils.HttpUtils;
import app.utils.MatchUtils;
import app.utils.TimeControl;
import com.google.gson.Gson;
import org.springframework.lang.NonNull;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

import static app.utils.TimeControl.*;

@Service
public class ChessComService {
    // TODO: make everything nullsafe

    private static final Gson gson = new Gson();
    private static final String ARCHIVES_ENDPOINT = "https://api.chess.com/pub/player/{username}/games/archives";

    private final Map<String, ChessComGame> chessComGameCache = new HashMap<>();

    public ChessComService() {
    }

    @NonNull
    public Map<TimeControl, List<Game>> getLatestGamesBetweenPlayers(@NonNull Match match, int maxAmountOfMonths) {
        Map<TimeControl, List<Game>> map = MatchUtils.createEmptyGamesMap();
        for (String archiveUrl : getLatestArchiveUrls(match.getPlayer1(), maxAmountOfMonths)) {
            List<ChessComGame> chessComGames = getChessComGames(archiveUrl).stream()
                    .sorted(Comparator.comparingLong(ChessComGame::getEndTime).reversed())
                    .filter(chessComGame -> ChessComUtils.isGameBetweenPlayers(chessComGame, match))
                    .filter(ChessComUtils::hasValidTimeControl)
                    .collect(Collectors.toList());
            chessComGames.forEach(chessComGame -> chessComGameCache.put(chessComGame.getUrl(), chessComGame));
            List<Game> games = chessComGames.stream()
                    .map(chessComGame -> getGame(chessComGame, match))
                    .collect(Collectors.toList());
            map.get(TEN_MINUTES).addAll(games.stream().filter(game -> game.getGameInfo().getTimeControl().equals("600")).collect(Collectors.toList()));
            map.get(FIVE_MINUTES).addAll(games.stream().filter(game -> game.getGameInfo().getTimeControl().equals("300")).collect(Collectors.toList()));
            map.get(THREE_MINUTES).addAll(games.stream().filter(game -> game.getGameInfo().getTimeControl().equals("180")).collect(Collectors.toList()));
            if (map.get(TEN_MINUTES).size() >= 2 && map.get(FIVE_MINUTES).size() >= 2 && map.get(THREE_MINUTES).size() >= 2) {
                break;
            }
        }

        for (TimeControl key : map.keySet()) {
            map.get(key).sort(Comparator.comparingLong(game -> game.getGameInfo().getEndTime()));
            int size = map.get(key).size();
            if (size > 2) map.put(key, map.get(key).subList(size - 2, size));
        }
        return map;
    }

    private List<ChessComGame> getChessComGames(String archiveUrl) {
        List<ChessComGame> list = new ArrayList<>();
        getChessComArchive(archiveUrl).ifPresent(chessComArchive -> list.addAll(chessComArchive.getGames()));
        return list;
    }

    private Optional<ChessComArchive> getChessComArchive(String archiveUrl) {
        String unpreparedJson = HttpUtils.getJson(archiveUrl);
        if (unpreparedJson == null) {
            return Optional.empty();
        }
        String preparedJson = ChessComUtils.getPreparedArchiveJson(unpreparedJson);
        return Optional.of(gson.fromJson(preparedJson, ChessComArchive.class));
    }

    private List<String> getLatestArchiveUrls(@NonNull Player player, int maxAmountOfMonths) {
        List<String> archiveUrls = getArchiveUrls(player);
        if (archiveUrls.size() >= maxAmountOfMonths) {
            archiveUrls = archiveUrls.subList(0, maxAmountOfMonths);
        }
        return archiveUrls;
    }

    private List<String> getArchiveUrls(@NonNull Player player) {
        String endPoint = ARCHIVES_ENDPOINT.replace("{username}", ChessComUtils.getUsernameForUrl(player));

        List<String> archiveUrls = gson.fromJson(HttpUtils.getJson(endPoint), ChessComArchiveList.class).getArchives();
        archiveUrls.sort(Collections.reverseOrder());
        return archiveUrls;
    }

    public Optional<Game> getGame(@NonNull String url, @NonNull Match match) {
        Optional<ChessComGame> chessComGame = getChessComGame(url, match);
        return chessComGame.map(game -> getGame(game, match));
    }

    @NonNull
    public Game getGame(@NonNull ChessComGame chessComGame, @NonNull Match match) {
        Game game = new Game();
        GameInfo gameInfo = new GameInfo();

        game.setMatch(match);
        game.setPlayer1IsWhite(chessComGame.getWhite().getUsername().equals(match.getPlayer1().getNickname()));
        game.setResult(ChessComUtils.getResult(chessComGame));
        game.setGameInfo(gameInfo);

        gameInfo.setChessComId(ChessComUtils.getGameId(chessComGame));
        gameInfo.setTimeControl(chessComGame.getTimeControl());
        gameInfo.setFen(chessComGame.getFen());
        gameInfo.setEndTime(chessComGame.getEndTime());
        gameInfo.setGame(game);
        gameInfo.setPgn(chessComGame.getPgn());
        gameInfo.setRated(chessComGame.isRated());
        gameInfo.setTimeClass(chessComGame.getTimeClass());
        gameInfo.setRules(chessComGame.getRules());
        gameInfo.setWhiteRating(chessComGame.getWhite().getRating());
        gameInfo.setBlackRating(chessComGame.getBlack().getRating());
        gameInfo.setWhiteResult(chessComGame.getWhite().getResult());
        gameInfo.setBlackResult(chessComGame.getBlack().getResult());

        return game;
    }

    public Optional<ChessComGame> getChessComGame(@NonNull Game game) {
        return getChessComGame(game, game.getMatch().getPlayer1())
                .or(() -> getChessComGame(game, game.getMatch().getPlayer2()));
    }

    private Optional<ChessComGame> getChessComGame(@NonNull Game game, Player player) {
        if (!(game.getMatch().getPlayer1().equals(player) || game.getMatch().getPlayer2().equals(player))) {
            throw new IllegalArgumentException("Player must be participating in Game!");
        }
        return getChessComGame(ChessComUtils.getGameURL(game.getGameInfo().getChessComId()), game.getMatch());
    }

    public Optional<ChessComGame> getChessComGame(@NonNull String url, Match match) {
        Optional<ChessComGame> gameFromCache = Optional.ofNullable(chessComGameCache.get(url));
        if (gameFromCache.isPresent()) {
            return gameFromCache;
        }

        for (Player player : Set.of(match.getPlayer1(), match.getPlayer2())) {
            for (String archiveUrl : getArchiveUrls(player)) {
                Optional<ChessComArchive> chessComArchive = getChessComArchive(archiveUrl);
                if (chessComArchive.isEmpty()) {
                    continue;
                }
                List<ChessComGame> chessComGames = chessComArchive.get().getGames();
                for (ChessComGame chessComGame : chessComGames) {
                    if (chessComGame.getUrl().equals(url)) {
                        return Optional.of(chessComGame);
                    }
                }
            }
        }
        return Optional.empty();
    }

}
