package app.data.service;

import app.data.bean.CalculatedMatch;
import app.data.entity.Matchday;
import app.data.entity.Season;
import app.data.repository.MatchdayRepository;
import app.navigation.NavigableService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.lang.NonNull;
import org.springframework.stereotype.Service;
import org.vaadin.artur.helpers.CrudService;

import java.util.Comparator;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class MatchdayService extends CrudService<Matchday, Integer> implements NavigableService<Matchday> {

    private final MatchdayRepository repository;

    private final MatchService matchService;

    public MatchdayService(@Autowired MatchdayRepository repository, MatchService matchService) {
        this.repository = repository;
        this.matchService = matchService;
    }

    @Override
    protected MatchdayRepository getRepository() {
        return repository;
    }

    @NonNull
    public List<Matchday> getMatchdaysSorted(@NonNull Season season) {
        return repository.findAll().stream()
                .filter(matchday -> matchday.getSeason().equals(season))
                .sorted(Comparator.comparingInt(Matchday::getNumber))
                .collect(Collectors.toList());
    }

    public Optional<Matchday> getFirstMatchday(@NonNull Season season) {
        List<Matchday> matchdays = getMatchdaysSorted(season);
        if (matchdays.isEmpty()) {
            return Optional.empty();
        }
        return Optional.of(matchdays.get(0));
    }

    @NonNull
    public List<Matchday> getMatchdaysWithActivitySorted(@NonNull Season season) {
        return repository.findAll().stream()
                .filter(matchday -> matchday.getSeason().equals(season))
                .filter(this::hasActivity)
                .sorted(Comparator.comparingInt(Matchday::getNumber))
                .collect(Collectors.toList());
    }

    public List<Matchday> getMatchdaysWithActivitySortedOrElseListWithOnlyFirstMatchday(@NonNull Season season) {
        List<Matchday> matchdaysWithActivitySorted = getMatchdaysWithActivitySorted(season);
        return matchdaysWithActivitySorted.isEmpty() ? List.of(getFirstMatchday(season).orElseThrow()) : matchdaysWithActivitySorted;
    }

    public boolean hasActivity(@NonNull Matchday matchday) {
        return matchday.getMatches().stream().mapToInt(match -> match.getGames().size()).sum() != 0;
    }

    @Override
    public Class<Matchday> getNavigableClass() {
        return Matchday.class;
    }

    public List<CalculatedMatch> getCalculatedMatches(Matchday matchday) {
        return matchService.getCalculatedMatches(matchday);
    }
}
