package app.navigation;

import app.components.label.ValidationLabel;
import app.utils.EntityStringUtils;
import com.vaadin.flow.component.AbstractField;
import com.vaadin.flow.component.HasValue;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.select.Select;
import com.vaadin.flow.router.BeforeEvent;
import com.vaadin.flow.router.HasUrlParameter;
import com.vaadin.flow.router.WildcardParameter;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

import java.util.*;

import static app.navigation.NavigationUtils.EDIT;

public abstract class Navigation implements HasUrlParameter<String> {
    protected String route;

    protected final List<Class<? extends Navigable>> navigableClasses = new ArrayList<>();

    protected final Map<Class<? extends Navigable>, NavigableService<? extends Navigable>> serviceMap = new HashMap<>();

    protected final Map<Class<? extends Navigable>, List<? extends Navigable>> listMap = new HashMap<>();

    protected final Map<Class<? extends Navigable>, Select<? extends Navigable>> selectMap = new HashMap<>();

    protected final List<Runnable> runnablesToBeRunAfterSelection = new ArrayList<>();

    protected final ValidationLabel validationLabel = new ValidationLabel();

    protected boolean editFlag = false;

    @SafeVarargs
    public Navigation(NavigableService<? extends Navigable>... services) {
        for (NavigableService<? extends Navigable> service : services) {
            initNavigable(service);
        }

        if (!navigableClasses.isEmpty()) {
            fillChildrenSelectWithData(null, navigableClasses.get(0));
        }
    }

    private <T extends Navigable> void initNavigable(NavigableService<T> service) {
        Class<T> clazz = service.getNavigableClass();

        navigableClasses.add(clazz);

        serviceMap.put(clazz, service);

        listMap.put(clazz, new ArrayList<>());

        Select<T> select = new Select<>();
        select.addValueChangeListener(selectValueChangeListener(clazz));
        select.setItemLabelGenerator(EntityStringUtils::getObjectString);
        selectMap.put(clazz, select);
    }

    public final void setRoute(String route) {
        this.route = route;
    }

    public final ValidationLabel getValidationLabel() {
        return validationLabel;
    }

    protected final String getRoute() {
        if (route != null) return route;
        throw new IllegalStateException("Route must be set!");
    }

    protected void doPostSelectionStuff() {
        validationLabel.setValid(true);
        updateUrl();
        runnablesToBeRunAfterSelection.forEach(Runnable::run);
    }

    protected abstract <PARENT extends Navigable, CHILD extends Navigable> List<CHILD> getChildren(@Nullable PARENT parent, @NonNull Class<CHILD> childClass);

    protected abstract <T extends Navigable> T getDefaultValue(Class<T> clazz);

    protected void updateUrl() {
        String[] params = new String[navigableClasses.size()];
        for (int i = 0; i < navigableClasses.size(); i++) {
            Select<? extends Navigable> select = selectMap.get(navigableClasses.get(i));
            if (select.getOptionalValue().isPresent()) {
                params[i] = EntityStringUtils.getObjectStringForURL(select.getValue());
            }
        }

        String wildcardParam = NavigationUtils.getWildcardParam(editFlag, params);
        UI.getCurrent().getPage().getHistory().pushState(null, String.format("%s/%s", getRoute(), wildcardParam));
    }

    public boolean editFlag() {
        return editFlag;
    }

    public void setEditFlag(boolean editFlag) {
        if (editFlag != this.editFlag) {
            this.editFlag = editFlag;
            updateUrl();
            runnablesToBeRunAfterSelection.forEach(Runnable::run);
        }
    }

    public void addRunnableToBeRunAfterSelection(Runnable runnable) {
        runnablesToBeRunAfterSelection.add(runnable);
    }

    protected <T extends Navigable> HasValue.ValueChangeListener<AbstractField.ComponentValueChangeEvent<Select<T>, T>> selectValueChangeListener(Class<T> clazz) {
        return event -> {
            if (event.getValue() != null) {
                if (!isLastClass(clazz)) {
                    Class<? extends Navigable> childClass = getChildClass(clazz);
                    fillChildrenSelectWithData(event.getValue(), childClass);
                    autoselect(childClass);
                    return;
                }
                doPostSelectionStuff();
            }
            // TODO: add else?
        };
    }

    private boolean isLastClass(@NonNull Class<? extends Navigable> clazz) {
        return navigableClasses.indexOf(clazz) == navigableClasses.size() - 1;
    }

    @Nullable
    private Class<? extends Navigable> getChildClass(@NonNull Class<? extends Navigable> clazz) {
        if (!navigableClasses.contains(clazz) || isLastClass(clazz)) return null;
        return navigableClasses.get(navigableClasses.indexOf(clazz) + 1);
    }

    @SuppressWarnings("unchecked")
    protected <CHILD extends Navigable, PARENT extends Navigable> void fillChildrenSelectWithData(PARENT parent, Class<CHILD> childClass) {
        List<CHILD> children = getChildren(parent, childClass);
        listMap.put(childClass, children);
        Select<CHILD> childSelect = (Select<CHILD>) selectMap.get(childClass);
        childSelect.setItems(children);
    }

    @SuppressWarnings("unchecked")
    protected <T extends Navigable> void autoselect(Class<T> clazz) {
        if (listMap.get(clazz).isEmpty()) {
            validationLabel.setText(String.format("No %ss in List!", clazz.getSimpleName()));
            validationLabel.setValid(false);
            return;
        }
        Select<T> childSelect = (Select<T>) selectMap.get(clazz);
        childSelect.setValue(getDefaultValue(clazz));
    }

    @Override
    public void setParameter(BeforeEvent event, @WildcardParameter String param) {
        String[] params = param.split("/");
        editFlag = params[params.length - 1].equals(EDIT);

        for (int i = 0; i < Math.min(params.length, navigableClasses.size()); i++) {
            navigateClass(navigableClasses.get(i), params[i]);
        }
    }

    @SuppressWarnings("unchecked")
    private <T extends Navigable> void navigateClass(Class<T> clazz, String param) {
        List<T> list = (List<T>) listMap.get(clazz);
        Select<T> select = (Select<T>) selectMap.get(clazz);
        Optional<T> navigable = NavigationUtils.getObjectFromParam(list, param);
        if (navigable.isPresent())
            select.setValue(navigable.get());
        else autoselect(clazz);
    }

    public String getWildcardParam() {
        String[] params = new String[navigableClasses.size()];
        for (int i = 0; i < navigableClasses.size(); i++) {
            params[i] = getParam(navigableClasses.get(i)).orElse(null);
        }
        return NavigationUtils.getWildcardParam(editFlag, params);
    }

    @SuppressWarnings("unchecked")
    private <T extends Navigable> Optional<String> getParam(Class<T> clazz) {
        Select<T> select = (Select<T>) selectMap.get(clazz);
        return select.getOptionalValue().map(EntityStringUtils::getObjectStringForURL);
    }
}