package app.views.matchday.components;

import app.data.bean.CalculatedMatch;
import app.data.entity.Matchday;
import app.data.entity.Season;
import app.navigation.matchday.MatchdayNavigation;
import app.navigation.matchday.components.button.NextMatchdayButton;
import app.navigation.matchday.components.button.PrevMatchdayButton;
import app.utils.ComponentUtils;
import app.utils.EntityStringUtils;
import app.utils.StringUtils;
import app.views.navigation.interfaces.ContentConfigurable;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.button.ButtonVariant;
import com.vaadin.flow.component.grid.ColumnTextAlign;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.grid.GridVariant;
import com.vaadin.flow.component.html.Div;
import com.vaadin.flow.component.html.Label;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.NoSuchElementException;

public class MatchdayCard extends Div implements ContentConfigurable {

    private final MatchdayNavigation matchdayNavigation;

    private final Calendar calendar = Calendar.getInstance();
    private final SimpleDateFormat dateFormat = new SimpleDateFormat("dd.MM.");

    private final HorizontalLayout header = new HorizontalLayout();
    private final HorizontalLayout headerLabelLayout = new HorizontalLayout();
    private final Grid<CalculatedMatch> grid = new Grid<>();

    public MatchdayCard(MatchdayNavigation matchdayNavigation) {
        this.matchdayNavigation = matchdayNavigation;

        addClassName("card");
        add(new VerticalLayout(header, grid));

        defineCalendar();

        defineHeader();
        defineGrid();
    }

    private void defineCalendar() {
        calendar.setFirstDayOfWeek(Calendar.MONDAY);
    }

    private void defineHeader() {
        header.add(new PrevMatchdayButton(this.matchdayNavigation), headerLabelLayout, new NextMatchdayButton(this.matchdayNavigation));
        header.setWidthFull();

        headerLabelLayout.addClassName("matchday-header-label-layout");
    }

    private void defineGrid() {
        Label headerPlayer1 = new Label("Player 1");
        headerPlayer1.addClassName("column-header");
        Label headerPlayer2 = new Label("Player 2");
        headerPlayer2.addClassName("column-header");
        Label headerResult = new Label("Result");
        headerResult.addClassName("column-header");

        grid.addComponentColumn(calculatedMatch -> ComponentUtils.getPlayerLabel(calculatedMatch.getPlayer1(), getSeason(calculatedMatch), true))
                .setHeader(headerPlayer1)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("13em")
                .setFlexGrow(1);

        grid.addColumn(calculatedMatch -> "vs.")
                .setHeader("vs.")
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("4em");

        grid.addComponentColumn(calculatedMatch -> ComponentUtils.getPlayerLabel(calculatedMatch.getPlayer2(), getSeason(calculatedMatch), true))
                .setHeader(headerPlayer2)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("13em")
                .setFlexGrow(1);

        grid.addColumn(this::getResultString)
                .setHeader(headerResult)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("6em");

        grid.addComponentColumn(this::createButton)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("4em");

        grid.setWidth("40em"); // TODO: find a way to set this dynamically based on column widths
        grid.setHeightByRows(true);

        grid.addThemeVariants(GridVariant.LUMO_NO_BORDER,
                GridVariant.LUMO_NO_ROW_BORDERS, GridVariant.LUMO_ROW_STRIPES);
    }

    @SuppressWarnings("DuplicatedCode") // TODO: get rid of duplicates
    private Button createButton(CalculatedMatch match) {
        Button button = new Button();
        button.addThemeVariants(ButtonVariant.LUMO_TERTIARY_INLINE);

        String seasonAndMatchdayParam = matchdayNavigation.getWildcardParam();
        String matchParam = EntityStringUtils.getMatchStringForURL(match.getMatch());
        String targetWildcardParam = String.format("match/%s%s/", seasonAndMatchdayParam, matchParam);

        if (match.getScore1() == 0 && match.getScore2() == 0) {
            button.setIcon(VaadinIcon.PENCIL.create());
            button.addClickListener(event -> UI.getCurrent().navigate(targetWildcardParam + "edit"));
            return button;
        }

        button.setIcon(VaadinIcon.EYE.create());
        button.addClickListener(event -> UI.getCurrent().navigate(targetWildcardParam));
        return button;
    }

    private Season getSeason(CalculatedMatch calculatedMatch) {
        return calculatedMatch.getMatch().getMatchday().getSeason();
    }

    private String getResultString(CalculatedMatch match) {
        return StringUtils.getResultString(":", match.getScore1(), match.getScore2());
    }

    @Override
    public void configureContent() {
        try {
            Matchday matchday = matchdayNavigation.getSelectedMatchday().orElseThrow();
            configureHeaderLabels(matchday);
            grid.setItems(matchdayNavigation.getMatchdayService().getCalculatedMatches(matchday));
        } catch (NoSuchElementException e) {
            removeAll();
            add(matchdayNavigation.getValidationLabel());
        }
    }

    private void configureHeaderLabels(Matchday matchday) {
        int week = matchday.getSeason().getWeekOfFirstMatchday() + matchday.getNumber(); // TODO: figure out why you have to omit -1 on the server!
        if (matchday.getNumber() > 13) { // TODO: Do this less dirty!
            week += 1;
        }
        int firstDay = matchday.getSeason().getFirstWeekdayOfMatchday();
        int lastDay = matchday.getSeason().getLastWeekdayOfMatchday();

        calendar.set(Calendar.WEEK_OF_YEAR, week);

        calendar.set(Calendar.DAY_OF_WEEK, firstDay);
        String startDate = dateFormat.format(calendar.getTime());

        calendar.set(Calendar.DAY_OF_WEEK, lastDay);
        String endDate = dateFormat.format(calendar.getTime());

        String mainText = String.format("Matchday %s", EntityStringUtils.getMatchdayString(matchday));
        String dateText = String.format("%s - %s", startDate, endDate);
        ComponentUtils.configureFormattedLabelWithParentheses(headerLabelLayout, mainText, dateText);
    }
}
