package app.views.player.components;

import app.data.bean.CalculatedMatch;
import app.data.entity.Player;
import app.data.entity.Season;
import app.navigation.player.PlayerNavigation;
import app.navigation.player.components.button.NextPlayerButton;
import app.navigation.player.components.button.PrevPlayerButton;
import app.utils.ComponentUtils;
import app.utils.EntityStringUtils;
import app.utils.StringUtils;
import app.views.navigation.interfaces.ContentConfigurable;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.button.ButtonVariant;
import com.vaadin.flow.component.grid.ColumnTextAlign;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.grid.GridVariant;
import com.vaadin.flow.component.html.Div;
import com.vaadin.flow.component.html.Label;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;

import java.util.NoSuchElementException;

public class PlayerCard extends Div implements ContentConfigurable {

    private final PlayerNavigation playerNavigation;

    private final HorizontalLayout header = new HorizontalLayout();
    private final HorizontalLayout headerLabelLayout = new HorizontalLayout();
    private final Grid<CalculatedMatch> grid = new Grid<>();

    private Player player;


    public PlayerCard(PlayerNavigation playerNavigation) {
        this.playerNavigation = playerNavigation;

        addClassName("card");
        add(new VerticalLayout(header, grid));

        defineHeader();
        defineGrid();
    }

    private void defineHeader() {
        header.add(new PrevPlayerButton(this.playerNavigation), headerLabelLayout, new NextPlayerButton(this.playerNavigation));
        header.setWidthFull();

        headerLabelLayout.addClassName("player-header-label-layout");
    }

    private void defineGrid() {
        Label headerMatchday = new Label("Matchday");
        headerMatchday.addClassName("column-header");
        Label headerOpponent = new Label("Opponent");
        headerOpponent.addClassName("column-header");
        Label headerResult = new Label("Result");
        headerResult.addClassName("column-header");

        grid.addColumn(calculatedMatch -> calculatedMatch.getMatch().getMatchday().getNumber())
                .setHeader(headerMatchday)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("7em");

        grid.addComponentColumn(calculatedMatch -> ComponentUtils.getPlayerLabel(getOpponent(calculatedMatch), getSeason(calculatedMatch), true))
                .setHeader(headerOpponent)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("13em");

        grid.addColumn(this::getResultString)
                .setHeader(headerResult)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("6em");

        grid.addComponentColumn(this::createButton)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("4em");

        grid.setWidth("31em"); // TODO: find a way to set this dynamically based on column widths
        grid.setHeightByRows(true);

        grid.addThemeVariants(GridVariant.LUMO_NO_BORDER,
                GridVariant.LUMO_NO_ROW_BORDERS, GridVariant.LUMO_ROW_STRIPES);
    }

    @SuppressWarnings("DuplicatedCode") // TODO; get rid of duplicates
    private Button createButton(CalculatedMatch match) {
        Button button = new Button();
        button.addThemeVariants(ButtonVariant.LUMO_TERTIARY_INLINE);

        String seasonParam = EntityStringUtils.getSeasonStringForURL(playerNavigation.getSelectedSeason().orElseThrow());
        String matchdayParam = EntityStringUtils.getMatchdayStringForURL(match.getMatch().getMatchday());
        String matchParam = EntityStringUtils.getMatchStringForURL(match.getMatch());

        String targetWildcardParam = String.format("match/%s/%s/%s/", seasonParam, matchdayParam, matchParam);

        if (match.getScore1() == 0 && match.getScore2() == 0) {
            button.setIcon(VaadinIcon.PENCIL.create());
            button.addClickListener(event -> UI.getCurrent().navigate(targetWildcardParam + "edit"));
            return button;
        }

        button.setIcon(VaadinIcon.EYE.create());
        button.addClickListener(event -> UI.getCurrent().navigate(targetWildcardParam));
        return button;
    }

    private Player getOpponent(CalculatedMatch match) {
        return match.getPlayer1().equals(player) ? match.getPlayer2() : match.getPlayer1();
    }

    private Season getSeason(CalculatedMatch calculatedMatch) {
        return calculatedMatch.getMatch().getMatchday().getSeason();
    }

    private String getResultString(CalculatedMatch match) {
        boolean isPlayer1 = match.getPlayer1().equals(player);
        double ownScore = isPlayer1 ? match.getScore1() : match.getScore2();
        double opponentScore = isPlayer1 ? match.getScore2() : match.getScore1();
        return StringUtils.getResultString(":", ownScore, opponentScore);
    }

    @Override
    public void configureContent() {
        try {
            player = playerNavigation.getSelectedPlayer().orElseThrow();
            Season season = playerNavigation.getSelectedSeason().orElseThrow();
            configureHeaderLabels(player, season);
            grid.setItems(playerNavigation.getPlayerService().getCalculatedMatchesSorted(player, season));
        } catch (NoSuchElementException e) {
            removeAll();
            add(playerNavigation.getValidationLabel());
        }
    }

    private void configureHeaderLabels(Player player, Season season) {
        ComponentUtils.configurePlayerLabel(headerLabelLayout, player, season);
    }
}
