package app.views.table.components;

import app.data.bean.PlayerForTable;
import app.data.entity.Matchday;
import app.data.service.PlayerService;
import app.navigation.matchday.MatchdayNavigation;
import app.navigation.matchday.components.button.NextMatchdayButton;
import app.navigation.matchday.components.button.PrevMatchdayButton;
import app.utils.ComponentUtils;
import app.utils.EntityStringUtils;
import app.utils.StringUtils;
import app.utils.VaadinUtils;
import app.views.navigation.interfaces.ContentConfigurable;
import com.vaadin.flow.component.Component;
import com.vaadin.flow.component.grid.ColumnTextAlign;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.grid.GridVariant;
import com.vaadin.flow.component.html.Div;
import com.vaadin.flow.component.html.Label;
import com.vaadin.flow.component.icon.Icon;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.orderedlayout.FlexComponent;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.function.ValueProvider;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.NoSuchElementException;

public class TableCard extends Div implements ContentConfigurable {

    private final MatchdayNavigation matchdayNavigation;

    private final PlayerService playerService;

    private final HorizontalLayout header = new HorizontalLayout();
    private final HorizontalLayout headerLabelLayout = new HorizontalLayout();
    private final Grid<PlayerForTable> grid = new Grid<>();

    public TableCard(MatchdayNavigation matchdayNavigation, @Autowired PlayerService playerService) {
        this.matchdayNavigation = matchdayNavigation;
        this.playerService = playerService;

        addClassName("card");
        add(new VerticalLayout(header, grid));

        defineHeader();
        defineGrid();
    }

    private void defineHeader() {
        header.add(new PrevMatchdayButton(this.matchdayNavigation), headerLabelLayout, new NextMatchdayButton(this.matchdayNavigation));
        header.setWidthFull();

        headerLabelLayout.addClassName("table-header-label-layout");
    }

    private void defineGrid() {
        // TODO: rounded corners for rows?

        Label headerPlace = new Label("Place");
        headerPlace.addClassName("important-table-column-header");

        Label headerPlayer = new Label("Player");
        headerPlayer.addClassName("important-table-column-header");

        Label headerMatchesPlayed = new Label("Played");

        Label headerMatchPoints = new Label("Points");
        headerMatchPoints.addClassName("important-table-column-header");

        Label headerWon = new Label("W");

        Label headerDrawn = new Label("D");

        Label headerLost = new Label("L");

        Label headerGames = new Label("Games");

        Label headerDiff = new Label("Diff");

        grid.addComponentColumn(this::getPlaceColumnItem)
                .setHeader(headerPlace)
                .setTextAlign(ColumnTextAlign.START)
                .setWidth("5em");

        grid.addComponentColumn(playerForTable -> ComponentUtils.getPlayerLabel(playerForTable.getPlayer(), matchdayNavigation.getSelectedSeason().orElseThrow(), false))
                .setHeader(headerPlayer)
                .setTextAlign(ColumnTextAlign.START)
                .setWidth("13em");

        grid.addColumn(PlayerForTable::getAmountOfMatches)
                .setHeader(headerMatchesPlayed)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("5em");

        grid.addColumn(VaadinUtils.getBoldStringRenderer(player -> String.valueOf(player.getMatchPoints())))
                .setHeader(headerMatchPoints)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("6em");

        grid.addColumn(PlayerForTable::getAmountOfMatchesWon)
                .setHeader(headerWon)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");

        grid.addColumn(PlayerForTable::getAmountOfMatchesDrawn)
                .setHeader(headerDrawn)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");

        grid.addColumn(PlayerForTable::getAmountOfMatchesLost)
                .setHeader(headerLost)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");

        grid.addColumn(this::getResultString)
                .setHeader(headerGames)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("6em");

        grid.addColumn(player -> StringUtils.getSignedString(player.getGamePointDiff()))
                .setHeader(headerDiff)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("5em");

        grid.setWidth("55em");
        grid.setHeightByRows(true);

        grid.addThemeVariants(GridVariant.LUMO_NO_BORDER,
                GridVariant.LUMO_NO_ROW_BORDERS, GridVariant.LUMO_ROW_STRIPES);
    }

    private Icon getPlaceDiffIcon(PlayerForTable playerForTable) {
        int placeDiff = playerForTable.getPlaceDiffToLastMatchday();
        if (placeDiff < 0) {
            Icon icon = VaadinIcon.CARET_DOWN.create();
            icon.setColor("#ca342d");
            return icon;
        }
        if (placeDiff > 0) {
            Icon icon = VaadinIcon.CARET_UP.create();
            icon.setColor("#96bc4b");
            return icon;
        }
        return VaadinIcon.MINUS.create();
    }

    private HorizontalLayout getPlaceColumnItem(PlayerForTable playerForTable) {
        Label label = new Label(playerForTable.getPlaceString());
        Icon icon = getPlaceDiffIcon(playerForTable);
        icon.setSize("1em");
        HorizontalLayout horizontalLayout = new HorizontalLayout(label, icon);
        horizontalLayout.setJustifyContentMode(FlexComponent.JustifyContentMode.BETWEEN);
        horizontalLayout.setAlignItems(FlexComponent.Alignment.CENTER);
        return horizontalLayout;
    }

    private String getResultString(PlayerForTable player) {
        return StringUtils.getResultString(":", player.getGamePointsForSelf(), player.getGamePointsForOpponents());
    }

    @Override
    public void configureContent() {
        try {
            Matchday matchday = matchdayNavigation.getSelectedMatchday().orElseThrow();

            String mainText = "Table";
            String matchdayText = String.format("Matchday %s", EntityStringUtils.getMatchdayString(matchday));
            ComponentUtils.configureFormattedLabelWithParentheses(headerLabelLayout, mainText, matchdayText);
            grid.setItems(playerService.getPlayersForTable(matchday));
        } catch (NoSuchElementException e) {
            removeAll();
            add(matchdayNavigation.getValidationLabel());
        }
    }
}
